import datetime

from django.db import models
from django.test import TestCase
from django.urls import reverse

from pinax.calendars.adapters import EventAdapter
from pinax.calendars.templatetags.pinax_calendars_tags import calendar, delta


class EventModel(models.Model):
    date = models.DateTimeField()


class Tests(TestCase):

    def test_delta_previous(self):
        """
        plus_year, plus_month = delta(date.year, date.month, 1)
        minus_year, minus_month = delta(date.year, date.month, -1)
        """
        self.assertEqual(delta(2016, 2, -1), (2016, 1))
        self.assertEqual(delta(2016, 1, -1), (2015, 12))

    def test_delta_next(self):
        """
        plus_year, plus_month = delta(date.year, date.month, 1)
        minus_year, minus_month = delta(date.year, date.month, -1)
        """
        self.assertEqual(delta(2016, 2, 1), (2016, 3))
        self.assertEqual(delta(2015, 12, 1), (2016, 1))

    def test_calendar_prev_next(self):
        events = EventAdapter(EventModel.objects.none())

        context = {}
        cal = calendar(context, events, datetime.datetime(2017, 2, 1))
        self.assertEqual(cal["prev"], reverse("monthly", args=[2017, 1]))
        self.assertEqual(cal["next"], reverse("monthly", args=[2017, 3]))



import calendar as cal
import datetime

from django import template
from django.utils import timezone

import pytz

register = template.Library()


def delta(year, month, d):
    mm = month + d
    yy = year
    if mm > 12:
        mm, yy = mm % 12, year + mm / 12
    elif mm < 1:
        mm, yy = 12 + mm, year - 1
    return int(yy), mm


@register.inclusion_tag("pinax/calendars/calendar.html", takes_context=True)
def calendar(context, events, date=None, tz=None, **kwargs):
    cal.setfirstweekday(cal.SUNDAY)

    if tz:
        today = timezone.localtime(timezone.now(), pytz.timezone(tz)).date()
    else:
        today = datetime.date.today()

    if date is None:
        date = today

    plus_year, plus_month = delta(date.year, date.month, 1)
    minus_year, minus_month = delta(date.year, date.month, -1)

    next = events.month_url(plus_year, plus_month, **kwargs)
    prev = events.month_url(minus_year, minus_month, **kwargs)

    events_by_day = events.events_by_day(date.year, date.month, tz, **kwargs)

    title = "%s %s" % (cal.month_name[date.month], date.year)

    matrix = cal.monthcalendar(date.year, date.month)
    grid = []
    for week in matrix:
        row = []
        for day in week:
            is_today = date.year == today.year and date.month == today.month and today.day == day
            if day:
                day_events = events_by_day.get(day, [])
                link = events.day_url(date.year, date.month,
                                      day, bool(day_events), **kwargs)
                row.append((day, day_events, link, is_today))
            else:
                row.append(None)
        grid.append(row)

    context.update({
        "title": title,
        "calendar_date": date,
        "prev": prev,
        "next": next,
        "grid": grid,
    })
    return context



from collections import defaultdict

from django.urls import reverse

import pytz


class EventAdapter(object):
    day_url_name = "daily"
    month_url_name = "monthly"
    date_field_name = "date"

    def __init__(self, queryset):
        self.events = queryset

    def day_url(self, year, month, day, has_event, **kwargs):
        if has_event:
            return reverse(self.day_url_name, args=[year, month, day])

    def month_url(self, year, month, **kwargs):
        return reverse(self.month_url_name, args=[year, month])

    def events_by_day(self, year, month, tz, **kwargs):
        days = defaultdict(list)
        query_args = {
            "{}__year".format(self.date_field_name): year,
            "{}__month".format(self.date_field_name): month
        }
        timezone = pytz.timezone(tz) if tz else pytz.utc
        for event in self.events.filter(**query_args).order_by(self.date_field_name):
            days[getattr(event, self.date_field_name).astimezone(timezone).day].append(event)
        return days



import datetime

from django.utils import timezone


class DailyMixin(object):

    month_kwarg_name = "month"
    year_kwarg_name = "year"
    day_kwarg_name = "day"

    def dispatch(self, request, *args, **kwargs):
        self.month = self.kwargs.get(self.month_kwarg_name)
        self.year = self.kwargs.get(self.year_kwarg_name)
        self.day = self.kwargs.get(self.day_kwarg_name)
        self.date = datetime.date(
            year=int(self.year),
            month=int(self.month),
            day=int(self.day)
        )
        return super(DailyMixin, self).dispatch(request, *args, **kwargs)


class MonthlyMixin(object):

    month_kwarg_name = "month"
    year_kwarg_name = "year"

    def dispatch(self, request, *args, **kwargs):
        self.month = self.kwargs.get(self.month_kwarg_name)
        self.year = self.kwargs.get(self.year_kwarg_name)
        if self.month and self.year:
            self.date = datetime.date(
                year=int(self.year),
                month=int(self.month),
                day=1
            )
        else:
            self.date = timezone.now().date()
        return super(MonthlyMixin, self).dispatch(request, *args, **kwargs)



from django.conf.urls import url
from django.views.generic import View

urlpatterns = [
    url(r"^(?P<year>\d{4})/(?P<month>\d{1,2})/$", View.as_view(), name="monthly"),
    url(r"^(?P<year>\d{4})/(?P<month>\d{1,2})/(?P<day>\d{1,2})/$", View.as_view(), name="daily"),
]
